@@ -1,4 +1,4 @@
-#   Copyright 2017-2019 typed_python Authors
+#   Copyright 2017-2021 typed_python Authors
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-__version__ = "0.2.0"
+__version__ = "0.2.1"
 
 # incremented every time the way we serialize a fully-typed
 # TP object changes. Consumers who only serialize fully-typed
@@ -26,14 +26,15 @@ __fully_typed_serialization_version__ = 1
 # or look them up in a codebase. Clients that serialize 'object' and 'type'
 # and who want a way to check that they are reading the correct protocol
 # can check this version.
-__untyped_serialization_version__ = 2
+__untyped_serialization_version__ = 4
 
 from typed_python.internals import (
     Member, Final, Function, UndefinedBehaviorException,
     makeNamedTuple, DisableCompiledCode, isCompiled, Held,
     typeKnownToCompiler,
     localVariableTypesKnownToCompiler,
-    checkOneOfType
+    checkOneOfType,
+    checkType
 )
 from typed_python._types import bytecount, refcount
 from typed_python.module import Module
@@ -46,7 +47,11 @@ from typed_python._types import (
     Alternative, Value, serialize, deserialize, serializeStream, deserializeStream,
     PointerTo, RefTo, Dict, validateSerializedObject, validateSerializedObjectStream, decodeSerializedObject,
     getOrSetTypeResolver, Set, Class, Type, BoundMethod,
-    TypedCell, pointerTo, refTo, copy, identityHash
+    TypedCell, pointerTo, refTo, copy, identityHash,
+    deepBytecount, deepcopy, deepcopyContiguous, totalBytesAllocatedInSlabs,
+    deepBytecountAndSlabs, Slab,
+    totalBytesAllocatedOnFreeStore,
+    ModuleRepresentation
 )
 import typed_python._types as _types
 import threading
