@@ -14,31 +14,27 @@
 
 import threading
 import os
+import time
 import types
 import typed_python.compiler.python_to_native_converter as python_to_native_converter
 import typed_python.compiler.llvm_compiler as llvm_compiler
 import typed_python
+from typed_python.compiler.runtime_lock import runtimeLock
 from typed_python.compiler.conversion_level import ConversionLevel
 from typed_python.compiler.compiler_cache import CompilerCache
 from typed_python.type_function import ConcreteTypeFunction
-from typed_python.compiler.type_wrappers.one_of_wrapper import OneOfWrapper
 from typed_python.compiler.type_wrappers.typed_tuple_masquerading_as_tuple_wrapper import TypedTupleMasqueradingAsTuple
 from typed_python.compiler.type_wrappers.named_tuple_masquerading_as_dict_wrapper import NamedTupleMasqueradingAsDict
 from typed_python.compiler.type_wrappers.python_typed_function_wrapper import PythonTypedFunctionWrapper
 from typed_python import Function, _types, Value
+from typed_python.compiler.merge_type_wrappers import mergeTypeWrappers
 
 _singleton = [None]
-_singletonLock = threading.Lock()
+_singletonLock = threading.RLock()
 
 typeWrapper = lambda t: python_to_native_converter.typedPythonTypeToTypeWrapper(t)
 
-
-def toInterpreterType(setOfTypes):
-    res = OneOfWrapper.mergeTypes(setOfTypes)
-    if res is None:
-        return None
-
-    return res.interpreterTypeRepresentation
+_resultTypeCache = {}
 
 
 class RuntimeEventVisitor:
@@ -49,6 +45,9 @@ class RuntimeEventVisitor:
     """
     def onNewFunction(
         self,
+        identifier,
+        functionConverter,
+        nativeFunction,
         funcName,
         funcCode,
         funcGlobals,
@@ -81,8 +80,14 @@ class PrintNewFunctionVisitor(RuntimeEventVisitor):
             # this should print out the fact that we compiled 'f'
             f()
     """
+    def __init__(self, short=False):
+        self.short = short
+
     def onNewFunction(
         self,
+        identifier,
+        functionConverter,
+        nativeFunction,
         funcName,
         funcCode,
         funcGlobals,
@@ -93,16 +98,33 @@ class PrintNewFunctionVisitor(RuntimeEventVisitor):
         variableTypes,
         conversionType
     ):
-        print("compiling ", funcName)
-        print("   inputs: ", inputTypes)
-        print("   output: ", outputType)
-        print("   vars: ")
+        if self.short:
+            print(
+                f"[complexity={len(str(nativeFunction)):8d}] ",
+                funcName,
+                "(" + ",".join([str(x.typeRepresentation) for x in inputTypes]) + ")",
+                "->",
+                outputType,
+            )
+        else:
+            print("compiling ", funcName)
+            print("   inputs: ", inputTypes)
+            print("   output: ", outputType)
+            print("   vars: ")
 
-        for varname, varVal in variableTypes.items():
-            if isinstance(varVal, type) and issubclass(varVal, Value):
-                print("        ", varname, " which is a Value(", varVal.Value, " of type ", type(varVal.Value), ")")
-            else:
-                print("        ", varname, varVal)
+            for varname, varVal in variableTypes.items():
+                if isinstance(varVal, type) and issubclass(varVal, Value):
+                    print(
+                        "        ",
+                        varname,
+                        "which is a Value(",
+                        varVal.Value,
+                        "of type",
+                        type(varVal.Value),
+                        ")"
+                    )
+                else:
+                    print("        ", varname, varVal)
 
 
 class CountCompilationsVisitor(RuntimeEventVisitor):
@@ -111,6 +133,9 @@ class CountCompilationsVisitor(RuntimeEventVisitor):
 
     def onNewFunction(
         self,
+        identifier,
+        functionConverter,
+        nativeFunction,
         funcName,
         funcCode,
         funcGlobals,
@@ -130,10 +155,6 @@ class Runtime:
         with _singletonLock:
             if _singleton[0] is None:
                 _singleton[0] = Runtime()
-
-            if os.getenv("TP_COMPILER_VERBOSE"):
-                _singleton[0].verboselyDisplayNativeCode()
-
         return _singleton[0]
 
     def __init__(self):
@@ -143,17 +164,27 @@ class Runtime:
             )
         else:
             self.compilerCache = None
-        self.llvm_compiler = llvm_compiler.Compiler()
+        self.llvm_compiler = llvm_compiler.Compiler(inlineThreshold=100)
         self.converter = python_to_native_converter.PythonToNativeConverter(
             self.llvm_compiler,
             self.compilerCache
         )
-        self.lock = threading.RLock()
+        self.lock = runtimeLock
         self.timesCompiled = 0
 
-    def verboselyDisplayNativeCode(self):
-        self.llvm_compiler.mark_converter_verbose()
-        self.llvm_compiler.mark_llvm_codegen_verbose()
+        if os.getenv("TP_COMPILER_VERBOSE"):
+            self.verbosityLevel = int(os.getenv("TP_COMPILER_VERBOSE"))
+            if self.verbosityLevel >= 2:
+                if self.verbosityLevel >= 3:
+                    self.addEventVisitor(PrintNewFunctionVisitor(False))
+                else:
+                    self.addEventVisitor(PrintNewFunctionVisitor(True))
+            if self.verbosityLevel >= 4:
+                self.llvm_compiler.mark_converter_verbose()
+            if self.verbosityLevel >= 5:
+                self.llvm_compiler.mark_llvm_codegen_verbose()
+        else:
+            self.verbosityLevel = 0
 
     def addEventVisitor(self, visitor: RuntimeEventVisitor):
         self.converter.addVisitor(visitor)
@@ -234,44 +265,60 @@ class Runtime:
 
         assert len(arguments) == len(overload.args)
 
-        with self.lock:
-            inputWrappers = []
+        try:
+            t0 = time.time()
+            t1 = None
+            t2 = None
+            defCount = self.converter.getDefinitionCount()
 
-            for i in range(len(arguments)):
-                inputWrappers.append(
-                    self.pickSpecializationTypeFor(overload.args[i], arguments[i], argumentsAreTypes)
+            with self.lock:
+                inputWrappers = []
+
+                for i in range(len(arguments)):
+                    inputWrappers.append(
+                        self.pickSpecializationTypeFor(overload.args[i], arguments[i], argumentsAreTypes)
+                    )
+
+                if any(x is None for x in inputWrappers):
+                    # this signature is unmatchable with these arguments.
+                    return None
+
+                self.timesCompiled += 1
+
+                callTarget = self.converter.convertTypedFunctionCall(
+                    functionType,
+                    overloadIx,
+                    inputWrappers,
+                    assertIsRoot=True
                 )
 
-            if any(x is None for x in inputWrappers):
-                # this signature is unmatchable with these arguments.
-                return None
+                callTarget = self.converter.demasqueradeCallTargetOutput(callTarget)
 
-            self.timesCompiled += 1
+                assert callTarget is not None
 
-            callTarget = self.converter.convertTypedFunctionCall(
-                functionType,
-                overloadIx,
-                inputWrappers,
-                assertIsRoot=True
-            )
+                wrappingCallTargetName = self.converter.generateCallConverter(callTarget)
 
-            callTarget = self.converter.demasqueradeCallTargetOutput(callTarget)
+                t1 = time.time()
+                self.converter.buildAndLinkNewModule()
+                t2 = time.time()
 
-            assert callTarget is not None
+                fp = self.converter.functionPointerByName(wrappingCallTargetName)
 
-            wrappingCallTargetName = self.converter.generateCallConverter(callTarget)
+                overload._installNativePointer(
+                    fp.fp,
+                    callTarget.output_type.typeRepresentation if callTarget.output_type is not None else type(None),
+                    [i.typeRepresentation for i in callTarget.input_types]
+                )
 
-            self.converter.buildAndLinkNewModule()
-
-            fp = self.converter.functionPointerByName(wrappingCallTargetName)
-
-            overload._installNativePointer(
-                fp.fp,
-                callTarget.output_type.typeRepresentation if callTarget.output_type is not None else type(None),
-                [i.typeRepresentation for i in callTarget.input_types]
-            )
-
-            return callTarget
+                return callTarget
+        finally:
+            if self.verbosityLevel > 0:
+                print(
+                    f"typed_python runtime spent {time.time()-t0:.3f} seconds "
+                    + (f"({t2 - t1:.3f})" if t2 is not None else "")
+                    + " adding " +
+                    f"{self.converter.getDefinitionCount() - defCount} functions."
+                )
 
     def compileClassDispatch(self, interfaceClass, implementingClass, slotIndex):
         with self.lock:
@@ -306,6 +353,11 @@ class Runtime:
         argTypes = [typeWrapper(a) for a in argTypes]
         kwargTypes = {k: typeWrapper(v) for k, v in kwargTypes.items()}
 
+        key = (type(funcObj), tuple(argTypes), tuple(kwargTypes.items()))
+
+        if key in _resultTypeCache:
+            return _resultTypeCache[key]
+
         possibleTypes = []
 
         for overloadIx in range(len(funcObj.overloads)):
@@ -327,7 +379,8 @@ class Runtime:
                     if callTarget is not None and callTarget.output_type is not None:
                         possibleTypes.append(callTarget.output_type)
 
-        return OneOfWrapper.mergeTypes(possibleTypes)
+        _resultTypeCache[key] = mergeTypeWrappers(possibleTypes)
+        return _resultTypeCache[key]
 
 
 def NotCompiled(pyFunc, returnTypeOverride=None):
