@@ -14,25 +14,19 @@
 
 import operator
 import sys
-from math import isfinite, trunc, floor, ceil
+from math import isfinite, trunc, floor, ceil, nan, inf
 from typed_python import (
-    Function, OneOf,
-    Bool,
-    Int8, Int16, Int32, Int64,
+    OneOf, ListOf, Tuple, Compiled, makeNamedTuple,
+    Int8, Int16, Int32, Float32,
     UInt8, UInt16, UInt32, UInt64,
-    Float32, Float64
 )
-from typed_python.type_promotion import computeArithmeticBinaryResultType
+from typed_python.type_promotion import (
+    computeArithmeticBinaryResultType, isSignedInt, isUnsignedInt, bitness
+)
 from typed_python import Entrypoint
-from typed_python.compiler.runtime import Runtime
 import unittest
 
 
-def Compiled(f):
-    f = Function(f)
-    return Runtime.singleton().compile(f)
-
-
 In = OneOf(int, float)
 Out = OneOf(int, float, bool)
 
@@ -113,33 +107,24 @@ ALL_OPERATIONS = [
 
 
 class TestArithmeticCompilation(unittest.TestCase):
-    def test_runtime_singleton(self):
-        self.assertTrue(Runtime.singleton() is Runtime.singleton())
-
     def test_compile_simple(self):
-        @Function
+        @Compiled
         def f(x: int) -> int:
             return x+x+x
 
-        r = Runtime.singleton()
-        r.compile(f.overloads[0])
-
         self.assertEqual(f(20), 60)
         self.assertEqual(f(10), 30)
 
     def test_in_to_out(self):
+        @Compiled
         def identity(x: In) -> Out:
             return x
 
-        compiled_identity = Runtime.singleton().compile(identity)
-
-        self.assertIsInstance(compiled_identity(0.5), float)
-        self.assertIsInstance(compiled_identity(0.0), float)
-        self.assertIsInstance(compiled_identity(1), int)
+        self.assertIsInstance(identity(0.5), float)
+        self.assertIsInstance(identity(0.0), float)
+        self.assertIsInstance(identity(1), int)
 
     def SKIPtest_binary_operators(self):
-        r = Runtime.singleton()
-
         failures = 0
         successes = 0
         for f in ALL_OPERATIONS:
@@ -157,7 +142,7 @@ class TestArithmeticCompilation(unittest.TestCase):
                 lvals = list(lvals) + [x / 3 for x in lvals]
                 rvals = list(rvals) + [x / 3 for x in rvals]
 
-            f_fast = r.compile(f)
+            f_fast = Compiled(f)
 
             for val1 in lvals:
                 for val2 in rvals:
@@ -180,9 +165,7 @@ class TestArithmeticCompilation(unittest.TestCase):
         self.assertEqual(failures, 0, successes)
 
     def checkFunctionOfIntegers(self, f):
-        r = Runtime.singleton()
-
-        f_fast = r.compile(f)
+        f_fast = Compiled(f)
 
         for i in range(100):
             self.assertEqual(f_fast(i), f(i))
@@ -251,16 +234,16 @@ class TestArithmeticCompilation(unittest.TestCase):
         self.assertEqual(toString(UInt8(10)), "10u8")
 
     def test_can_compile_register_builtins(self):
-        registerTypes = [Bool, Int8, Int16, Int32, Int64, UInt8, UInt16, UInt32, UInt64, Float32, Float64]
+        registerTypes = [bool, Int8, Int16, Int32, int, UInt8, UInt16, UInt32, UInt64, Float32, float]
 
         def suitable_range(t):
-            if t in [Bool]:
+            if t in [bool]:
                 return [0, 1]
-            elif t.IsUnsignedInt:
+            elif isUnsignedInt(t):
                 return [0, 5, 10, 15, 50, 100, 150]
-            elif t.IsSignedInt:
+            elif isSignedInt(t):
                 return [0, 5, 10, 15, 50, 100, -5, -10, -15, -50, -100]
-            elif t in [Float32, Float64]:
+            elif t in [Float32, float]:
                 return [x / 2.0 for x in range(-100, 100)]
 
         for T in registerTypes:
@@ -309,11 +292,33 @@ class TestArithmeticCompilation(unittest.TestCase):
                     r2 = c_op(T(v))
                     self.assertEqual(r1, r2)
                     # note that types are necessarily different sometimes,
-                    # e.g. round(Float64(1), 0) returns int when interpreted,
-                    # but Float64 when compiled
+                    # e.g. round(float(1), 0) returns int when interpreted,
+                    # but float when compiled
+
+    def test_can_call_types_with_no_args(self):
+        @Entrypoint
+        def makeEmpty(T):
+            return T()
+
+        self.assertEqual(makeEmpty(int), 0)
+        self.assertEqual(makeEmpty(float), 0.0)
+        self.assertEqual(makeEmpty(bool), False)
+        self.assertEqual(makeEmpty(str), "")
+
+    def test_not_on_float(self):
+        @Entrypoint
+        def doit(f):
+            if not f:
+                return "its false"
+            return "its true"
+
+        self.assertEqual(doit(1.0), "its true")
+        self.assertEqual(doit(0.0), "its false")
 
     def test_can_compile_register_operations(self):
-        registerTypes = [Bool, Int8, Int16, Int32, Int64, UInt8, UInt16, UInt32, UInt64, Float32, Float64]
+        failed = False
+
+        registerTypes = [bool, Int8, Int16, Int32, int, UInt8, UInt16, UInt32, UInt64, Float32, float]
 
         def result_or_exception(f, *p):
             try:
@@ -322,7 +327,7 @@ class TestArithmeticCompilation(unittest.TestCase):
                 return "exception"
 
         def equal_result(a, b):
-            if (type(a) is float or (hasattr(a, "IsFloat") and a.IsFloat)):
+            if type(a) in (float, Float32):
                 epsilon = float(1e-6)
                 if a < 1e-32:  # these results happen to be from calculations that magnify errors
                     epsilon = float(1e-5)
@@ -333,20 +338,20 @@ class TestArithmeticCompilation(unittest.TestCase):
             return a == b
 
         def signed_overflow(T, v):
-            return T.IsUnsignedInt and (v >= 2**(T.Bits - 1) or v < -2**(T.Bits - 1))
+            return isUnsignedInt(T) and (v >= 2**(bitness(T) - 1) or v < -2**(bitness(T) - 1))
 
         def suitable_range(t):
-            if t in [Bool]:
+            if t in [bool]:
                 return [0, 1]
-            elif t.IsUnsignedInt:
-                return [0, 1, 2, (1 << (t.Bits // 4)) - 1, (1 << (t.Bits // 2)) - 1, (1 << t.Bits) - 1]
-            elif t.IsSignedInt:
-                return [0, 1, 2, (1 << (t.Bits // 2 - 1)) - 1, (1 << (t.Bits - 1)) - 1,
-                        -1, -2, -(1 << (t.Bits // 2 - 1)), -(1 << (t.Bits - 1))]
+            elif isUnsignedInt(t):
+                return [0, 1, 2, (1 << (bitness(t) // 4)) - 1, (1 << (bitness(t) // 2)) - 1, (1 << bitness(t)) - 1]
+            elif isSignedInt(t):
+                return [0, 1, 2, (1 << (bitness(t) // 2 - 1)) - 1, (1 << (bitness(t) - 1)) - 1,
+                        -1, -2, -(1 << (bitness(t) // 2 - 1)), -(1 << (bitness(t) - 1))]
             elif t in [Float32]:
                 return [0.0, 1.0/3.0, 0.5, 1.0, 1.5, 2.0, (2 - 1 / (2**23)) * 2**127,
                         -1.0/3.0, -0.5, -1.0, -1.5, -2.0, -(2 - 1 / (2**23)) * 2**127]
-            elif t in [Float64]:
+            elif t in [float]:
                 return [0.0, 1e-16, 9.876e-16, 1.0/3.0, 0.5, 1.0, 1.5, 10.0/7.0, 2.0, 3.0, 10.0/3.0, 100.0/3.0, sys.float_info.max,
                         -1e-16, -9.876e-16, -1.0/3.0, -0.5, -1.0, -10.0/7.0, -1.5, -2.0, -3.0, -10.0/3.0, -100.0/3.0, -sys.float_info.max]
 
@@ -366,9 +371,9 @@ class TestArithmeticCompilation(unittest.TestCase):
             def abs_(x: T):
                 return abs(x)
 
-            if T is not Bool:
+            if T is not bool:
                 self.assertEqual(T(1) + T(2), T(3))
-            if T in [Bool]:
+            if T in [bool]:
                 suitable_ops = [not_]
             else:
                 suitable_ops = [invert, neg, pos, abs_]
@@ -386,27 +391,27 @@ class TestArithmeticCompilation(unittest.TestCase):
 
                 for v in suitable_range(T):
                     comparable = True
-                    native_result = result_or_exception(native_op, v)
-                    if T.IsSignedInt:
-                        if native_result >= 2**(T.Bits - 1) or native_result < -2**(T.Bits - 1):
+                    interpreterResult = result_or_exception(native_op, v)
+                    if isSignedInt(T):
+                        if interpreterResult >= 2**(bitness(T) - 1) or interpreterResult < -2**(bitness(T) - 1):
                             comparable = False
-                    elif T.IsUnsignedInt:
-                        if native_result >= 2**T.Bits or native_result < 0:
+                    elif isUnsignedInt(T):
+                        if interpreterResult >= 2**bitness(T) or interpreterResult < 0:
                             comparable = False
-                    typed_result = result_or_exception(op, T(v))
-                    if typed_result == NotImplemented:
-                        typed_result = "exception"
-                    compiled_result = result_or_exception(compiled_op, T(v))
+                    typedPythonResult = result_or_exception(op, T(v))
+                    if typedPythonResult == NotImplemented:
+                        typedPythonResult = "exception"
+                    compilerResult = result_or_exception(compiled_op, T(v))
 
                     self.assertEqual(
-                        type(typed_result), type(compiled_result),
-                        (T, type(typed_result), type(compiled_result), op.__name__)
+                        type(typedPythonResult), type(compilerResult),
+                        (T, type(typedPythonResult), type(compilerResult), op.__name__)
                     )
                     if comparable:
-                        self.assertTrue(equal_result(type(compiled_result)(typed_result), compiled_result),
-                                        (T, op.__name__, typed_result, compiled_result))
-                        self.assertTrue(equal_result(type(compiled_result)(native_result), compiled_result),
-                                        (T, op.__name__, native_result, compiled_result))
+                        self.assertTrue(equal_result(type(compilerResult)(typedPythonResult), compilerResult),
+                                        (T, op.__name__, typedPythonResult, compilerResult))
+                        self.assertTrue(equal_result(type(compilerResult)(interpreterResult), compilerResult),
+                                        (T, op.__name__, interpreterResult, compilerResult))
 
         for T1 in registerTypes:
             for T2 in registerTypes:
@@ -461,13 +466,15 @@ class TestArithmeticCompilation(unittest.TestCase):
                 def pow(x: T1, y: T2):
                     return x ** y
 
-                if T1 in [Bool] or T2 in [Bool]:
+                if T1 in [bool] or T2 in [bool]:
                     suitable_ops = [bitand, bitor, bitxor]
                 else:
                     suitable_ops = [
                         add, sub, mul, div, mod, floordiv,
                         less, greater, lessEq, greaterEq, neq,
-                        bitand, bitor, bitxor, lshift, rshift, pow
+                        bitand, bitor, bitxor,
+                        lshift,
+                        rshift, pow
                     ]
 
                 typed_to_native_op = {
@@ -498,27 +505,30 @@ class TestArithmeticCompilation(unittest.TestCase):
                         for v2 in suitable_range(T2):
                             comparable = True
                             T_result = computeArithmeticBinaryResultType(T1, T2)
-                            if T_result.IsSignedInt:
+                            if isSignedInt(T_result):
                                 if signed_overflow(T1, v1) or signed_overflow(T2, v2):
                                     comparable = False
-                            if (op is lshift and v1 != 0 and v2 > 1024) \
+
+                            if (op in (lshift, rshift) and v2 > 1024) \
                                     or (op is pow and (v1 > 1 or v1 < -1) and v2 > 1024):
-                                native_result = "exception"
+                                interpreterResult = "expected to disagree"
                             else:
-                                native_result = result_or_exception(native_op, v1, v2)
+                                interpreterResult = result_or_exception(native_op, v1, v2)
+
                                 T_result = computeArithmeticBinaryResultType(T1, T2)
-                                if native_result == "exception":
+
+                                if interpreterResult == "exception":
                                     pass
-                                elif type(native_result) is complex:
-                                    native_result = "exception"
-                                elif T_result.IsSignedInt:
-                                    if native_result >= 2**(T_result.Bits - 1) or native_result < -2**(T_result.Bits - 1):
+                                elif type(interpreterResult) is complex:
+                                    interpreterResult = "expected to disagree"
+                                elif isSignedInt(T_result):
+                                    if interpreterResult >= 2**(bitness(T_result) - 1) or interpreterResult < -2**(bitness(T_result) - 1):
                                         comparable = False
-                                elif T_result.IsUnsignedInt:
-                                    if native_result >= 2**T_result.Bits or native_result < 0:
+                                elif isUnsignedInt(T_result):
+                                    if interpreterResult >= 2**bitness(T_result) or interpreterResult < 0:
                                         comparable = False
-                                elif T_result.IsFloat:
-                                    if native_result >= 2**128 or native_result <= -2**128:
+                                elif T_result in (float, Float32):
+                                    if interpreterResult >= 2**128 or interpreterResult <= -2**128:
                                         comparable = False
 
                             native_comparable = comparable
@@ -526,47 +536,191 @@ class TestArithmeticCompilation(unittest.TestCase):
                                 # can't expect Float32 mod and floordiv to match native calculations
                                 # typed should still match compiled
                                 native_comparable = False
-                            if ((T1 is Float32 and T2 is Float64) or (T1 is Float64 and T2 is Float32)) \
+                            if ((T1 is Float32 and T2 is float) or (T1 is float and T2 is Float32)) \
                                     and op in [less, greater, lessEq, greaterEq, neq]:
                                 native_comparable = False
 
-                            if T1 is Int64 and T2 is Int64 and \
+                            if T1 is int and T2 is int and \
                                     ((op is lshift and v1 != 0 and v2 > 1024) or
                                      (op is pow and (v1 > 1 or v1 < -1) and v2 > 1024)):
-                                typed_result = "exception"
+                                typedPythonResult = "expected to disagree"
                             else:
-                                typed_result = result_or_exception(op, T1(v1), T2(v2))
-                                if type(typed_result) in [float, Float32, Float64] and not isfinite(typed_result):
-                                    typed_result = "exception"
-                                if type(typed_result) is complex:
-                                    typed_result = "exception"
-                                if op is pow and type(typed_result) is int:
-                                    typed_result = float(typed_result)
+                                typedPythonResult = result_or_exception(op, T1(v1), T2(v2))
+                                if type(typedPythonResult) in [float, Float32, float] and not isfinite(typedPythonResult):
+                                    typedPythonResult = "expected to disagree"
+                                if type(typedPythonResult) is complex:
+                                    typedPythonResult = "expected to disagree"
+                                if op is pow and type(typedPythonResult) is int:
+                                    typedPythonResult = float(typedPythonResult)
 
-                            compiled_result = result_or_exception(compiled_op, T1(v1), T2(v2))
-                            if type(compiled_result) in [float, Float32, Float64] and not isfinite(compiled_result):
-                                compiled_result = "exception"
+                            compilerResult = result_or_exception(compiled_op, T1(v1), T2(v2))
+
+                            if type(compilerResult) in [float, Float32, float] and not isfinite(compilerResult):
+                                compilerResult = "expected to disagree"
 
                             # for debugging
-                            # if type(typed_result) != type(compiled_result):
-                            #     print("type mismatch", type(typed_result).__name__, type(compiled_result).__name__)
-                            # if type(typed_result) == type(compiled_result):
-                            #     if comparable and not equal_result(typed_result, compiled_result):
+                            # if type(typedPythonResult) != type(compilerResult):
+                            #     print("type mismatch", type(typedPythonResult).__name__, type(compilerResult).__name__)
+                            # if type(typedPythonResult) == type(compilerResult):
+                            #     if comparable and not equal_result(typedPythonResult, compilerResult):
                             #         print("result mismatch")
                             # try:
-                            #     if native_comparable and not equal_result(type(compiled_result)(native_result), compiled_result):
+                            #     if native_comparable and not equal_result(type(compilerResult)(interpreterResult), compilerResult):
                             #         print("mismatch")
                             # except Exception:
                             #     print("mismatch exception")
 
-                            self.assertEqual(
-                                type(typed_result), type(compiled_result),
-                                (T1, T2, type(typed_result), type(compiled_result), op.__name__)
-                            )
+                            if type(typedPythonResult) != type(compilerResult):
+                                print(
+                                    f"interpreter and compiler types for {op.__name__} on arguments of type "
+                                    f"{T1} and {T2} were not the same. the interpreter gave {type(typedPythonResult)} but"
+                                    f"compiler gave {type(compilerResult)}"
+                                )
 
-                            if comparable:
-                                self.assertTrue(equal_result(type(compiled_result)(typed_result), compiled_result),
-                                                (T1, T2, op.__name__, typed_result, compiled_result))
-                            if native_comparable:
-                                self.assertTrue(equal_result(type(compiled_result)(native_result), compiled_result),
-                                                (T1, T2, op.__name__, native_result, compiled_result))
+                                failed = True
+
+                            if "expected to disagree" not in (typedPythonResult, compilerResult, interpreterResult):
+                                if (comparable and not equal_result(type(compilerResult)(typedPythonResult), compilerResult) or
+                                        native_comparable and not equal_result(type(compilerResult)(interpreterResult), compilerResult)):
+                                    print(
+                                        f"results for {op.__name__} on arguments "
+                                        f"{v1} of type {T1} and {v2} of type {T2} do not agree: "
+                                        f"interpreter gives {interpreterResult}, "
+                                        f"typed_python gave {typedPythonResult}, and "
+                                        f"the compiler gave {compilerResult}"
+                                    )
+
+                                    failed = True
+
+        self.assertFalse(failed)
+
+    def test_int_of_nan(self):
+        @Entrypoint
+        def f(x):
+            return int(x)
+
+        with self.assertRaisesRegex(Exception, "NaN"):
+            f(nan)
+
+        with self.assertRaisesRegex(Exception, "infinity"):
+            f(inf)
+
+    def test_mod_constant_in_tuple(self):
+        @Entrypoint
+        def rf(row):
+            return row.x % 1.0
+
+        self.assertEqual(rf(makeNamedTuple(x=1.0)), 0.0)
+
+    def test_pow_with_bad_inputs(self):
+        @Entrypoint
+        def callPow(x, y):
+            return x ** y
+
+        with self.assertRaises(ZeroDivisionError):
+            callPow(0.0, -1.0)
+
+        with self.assertRaises(ZeroDivisionError):
+            callPow(0, -1)
+
+    def test_floordiv_with_bad_inputs(self):
+        @Entrypoint
+        def callFloordiv(x, y):
+            return x // y
+
+        with self.assertRaises(ZeroDivisionError):
+            callFloordiv(1.0, 0.0)
+
+        with self.assertRaises(ZeroDivisionError):
+            callFloordiv(1, 0)
+
+    def test_shift_with_bad_inputs(self):
+        @Entrypoint
+        def callShift(x, y):
+            return x << y
+
+        with self.assertRaises(ValueError):
+            callShift(1, -2)
+
+        with self.assertRaises(OverflowError):
+            callShift(1, 2048)
+
+    def test_formatting_with_format_strings_works(self):
+        @Entrypoint
+        def format(x):
+            return f"{x:.6g}"
+
+        assert format(1.0) == f"{1.0:.6g}"
+
+    def test_formatting_strings_in_loop(self):
+        @Entrypoint
+        def format(a, b, c, d):
+            res = ListOf(str)()
+            for i in range(10000):
+                res.append(f"{i} {a} {b} {c} {d}")
+            return res
+
+        format(1, 2, 3, 4)
+
+    def test_string_joining_in_loop(self):
+        @Entrypoint
+        def format(a, b, c, d):
+            res = ListOf(str)()
+            for i in range(10000):
+                l = ListOf(str)()
+                l.append(str(a))
+                l.append(str(b))
+                l.append(str(c))
+                res.append("".join(l))
+            return res
+
+        format(1, 2, 3, 4)
+
+    def test_unary_ops_on_constants(self):
+        @Entrypoint
+        def sliceAtPositiveZero(x):
+            return x[+0]
+
+        self.assertIs(sliceAtPositiveZero.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, int)
+
+        @Entrypoint
+        def sliceAtNegativeZero(x):
+            return x[-0]
+
+        self.assertIs(sliceAtNegativeZero.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, int)
+
+        @Entrypoint
+        def sliceAtInvertZero(x):
+            return x[~0]
+
+        self.assertIs(sliceAtInvertZero.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, float)
+
+        @Entrypoint
+        def sliceAtNotZero(x):
+            return x[int(not 0)]
+
+        self.assertIs(sliceAtNotZero.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, float)
+
+        @Entrypoint
+        def sliceAtPositiveZeroFloat(x):
+            return x[int(+0.0)]
+
+        self.assertIs(sliceAtPositiveZeroFloat.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, int)
+
+        @Entrypoint
+        def sliceAtNegativeZeroFloat(x):
+            return x[int(-0.0)]
+
+        self.assertIs(sliceAtNegativeZeroFloat.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, int)
+
+        @Entrypoint
+        def sliceAtNotZeroFloat(x):
+            return x[int(not 0.0)]
+
+        self.assertIs(sliceAtNotZeroFloat.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, float)
+
+        @Entrypoint
+        def sliceAtNotFalse(x):
+            return x[int(not False)]
+
+        self.assertIs(sliceAtNotZeroFloat.resultTypeFor(Tuple(int, float)).interpreterTypeRepresentation, float)
