@@ -406,6 +406,11 @@ class ListOfWrapper(TupleOrListOfWrapper):
             listInst.nonref_expr.ElementPtrIntegers(0, 4).store(
                 runtime_functions.realloc.call(
                     listInst.nonref_expr.ElementPtrIntegers(0, 4).load(),
+                    # original bytecount
+                    listInst.nonref_expr.ElementPtrIntegers(0, 3).load().cast(native_ast.Int64).mul(
+                        self.underlyingWrapperType.getBytecount()
+                    ),
+                    # new bytecount
                     countInst.nonref_expr.mul(self.underlyingWrapperType.getBytecount())
                 )
             )
@@ -459,8 +464,6 @@ class ListOfWrapper(TupleOrListOfWrapper):
         if item is None:
             return None
 
-        item = item.refToHeld()
-
         item = item.convert_to_type(self.underlyingWrapperType, ConversionLevel.ImplicitContainers)
 
         if item is None:
