@@ -12,13 +12,14 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-import threading
 import logging
 
 from types import FunctionType
+from typed_python.compiler.runtime_lock import runtimeLock
 from typed_python._types import Forward, ListOf, TupleOf, Dict, ConstDict
+import typed_python
+
 
-_typeFunctionLock = threading.RLock()
 _type_to_typefunction = {}
 
 
@@ -82,17 +83,30 @@ class ConcreteTypeFunction:
             return tuple(sorted([(k, self.mapArg(v)) for k, v in arg.items()]))
 
         if isinstance(arg, FunctionType):
-            return self.mapArg(arg())
+            return self.mapArg(typed_python.Function(arg))
+
+        if isinstance(arg, typed_python._types.Function):
+            return arg
 
         raise TypeError("Instance of type '%s' is not a valid argument to a type function" % type(arg))
 
     def __call__(self, *args, **kwargs):
-        with _typeFunctionLock:
-            args = tuple(self.mapArg(a) for a in args)
-            kwargs = tuple(sorted([(k, self.mapArg(v)) for k, v in kwargs.items()]))
+        args = tuple(self.mapArg(a) for a in args)
+        kwargs = tuple(sorted([(k, self.mapArg(v)) for k, v in kwargs.items()]))
 
-            key = (args, kwargs)
+        key = (args, kwargs)
 
+        if key in self._memoForKey:
+            res = self._memoForKey[key]
+            if isinstance(res, Exception):
+                raise res
+
+            if getattr(res, '__typed_python_category__', None) != 'Forward':
+                # only return fully resolved TypeFunction values without
+                # locking.
+                return res
+
+        with runtimeLock:
             if key in self._memoForKey:
                 res = self._memoForKey[key]
                 if isinstance(res, Exception):
@@ -109,13 +123,13 @@ class ConcreteTypeFunction:
 
                 forward.define(resultType)
 
-                self._memoForKey[key] = resultType
-
                 _type_to_typefunction.pop(forward)
 
                 if resultType not in _type_to_typefunction:
                     _type_to_typefunction[resultType] = (self, key)
 
+                self._memoForKey[key] = resultType
+
                 return resultType
             except Exception as e:
                 self._memoForKey[key] = e
